import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Ant is a creature looking for food in the Arena. It moves randomly until it finds food
 * which it will take home.
 * 
 * @author Pavel Soukup 
 * @version v1
 */
public class Ant extends Actor
{
    private Anthill home;
    private boolean hasFood = false;
    private boolean smellsFood = false;
    private static int SMELL_DISTANCE = 25;
    private int smellDistance = SMELL_DISTANCE;
    private int stepSize = 2;

    /**
     * Constructor for creating an ant with an assigned anthill and random orientation
     */
    public Ant(Anthill anthill)
    {
        home = anthill;
        setRotation(Greenfoot.getRandomNumber(360));
    }

    /**
     * Act - do whatever the Ant wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {

        if(hasFood){
            goTowards(home);
            checkHome();
        } else {
            Actor food = smellFood();
            if(food!=null){
                goTowards(food);

                Actor closeFood;
                closeFood = getOneObjectAtOffset(0,0,HoneyDrop.class);

                if(closeFood!=null){
                    pickUpFood((HoneyDrop)closeFood);
                }

            } else {
                if(atWorldEdge()){
                    turn(180);
                } else {
                    turn(10-Greenfoot.getRandomNumber(21)); //otoc nahodne v rozmezi -10 az 10
                }
                move(stepSize);
            }
        }
    }

    /**
     * Return TRUE if ant is at the worlds edge
     */
    public boolean atWorldEdge()
    {
        if(getX() < 10 || getX() > getWorld().getWidth() - 10){ return true;}
        if(getY() < 10 || getY() > getWorld().getHeight() - 10){ return true;}
        return false;
    }

    /**
     * Smell food finds a closest food pile in smellDistance and returns it.
     */
    public Actor smellFood()
    {
        Actor closest = null; //empty closest food reference
        if(!getObjectsInRange(smellDistance, HoneyDrop.class).isEmpty()){
            for(Object obj: getObjectsInRange(smellDistance, HoneyDrop.class)){
                Actor food = (Actor) obj;
                int foodDist = (int) Math.sqrt((Math.pow(food.getX()-getX(),2)+Math.pow(food.getY()-getY(),2)));

                if(closest == null || foodDist < smellDistance){ //if distance is smaller than smellDistance OR there is NONE
                    closest = food;
                    smellDistance = foodDist; //decrease smell distance for next time to avoid smelling multiple sources
                }
            }

        }
        return(closest);
    }

    /**
     * Makes ant go in a direction.
     */
    public void goTowards(Actor target)
    {
        int x = target.getX();
        int y = target.getY();
        if(getX()==x&&getY()==y){
            //rest, you are there
        } else {
            turnTowards(x,y);
            move(stepSize);
        }
    }

    /**
     * An example of a method - replace this comment with your own
     */
    public void pickUpFood(HoneyDrop food)
    {
        hasFood = true;
        food.takeFood();
        setImage("ant-with-food.png");
    }

    /**
     * Checks if ant is at home
     */
    public void checkHome()
    {
        if(Math.abs(getX() - home.getX()) < 2*stepSize && 
        Math.abs(getY() - home.getY()) < 2*stepSize){
            dropFood();
            smellDistance = SMELL_DISTANCE;
        }
    }

    /**
     * Drops food
     */
    public void dropFood()
    {
        hasFood = false;
        setImage("ant.png");
    }

}
